@@ -1,11 +1,12 @@
 #![warn(missing_docs)]
 #![deny(missing_docs)]
+#![allow(unused_imports)]
 
 //! A wrapper for the Botan cryptography library
 
-#![cfg_attr(feature = "no-std", no_std)]
+#![cfg_attr(not(feature = "std"), no_std)]
 
-#[cfg(feature = "no-std")]
+#[cfg(not(feature = "std"))]
 #[macro_use]
 extern crate alloc;
 
@@ -95,9 +96,11 @@ macro_rules! botan_bool_in_rc {
     }};
 }
 
+mod asn1;
 mod bcrypt;
 mod block;
 mod cipher;
+mod ec_group;
 mod fpe;
 mod hash;
 mod kdf;
@@ -108,41 +111,40 @@ mod mp;
 mod otp;
 mod pbkdf;
 mod pk_ops;
+
 mod pubkey;
 mod rng;
 mod utils;
 mod version;
 mod x509_cert;
 mod x509_crl;
-
-#[cfg(feature = "botan3")]
-mod pk_ops_kem;
-
-#[cfg(feature = "botan3")]
 mod zfec;
 
-pub use crate::mp::*;
-pub use crate::rng::*;
-pub use crate::utils::*;
+pub use asn1::*;
 pub use bcrypt::*;
 pub use block::*;
 pub use cipher::*;
+pub use ec_group::*;
 pub use fpe::*;
 pub use hash::*;
 pub use kdf::*;
 pub use keywrap::*;
 pub use mac::*;
 pub use memutils::*;
+pub use mp::*;
 pub use otp::*;
 pub use pbkdf::*;
 pub use pk_ops::*;
 pub use pubkey::*;
+pub use rng::*;
+pub use utils::*;
 pub use version::*;
 pub use x509_cert::*;
 pub use x509_crl::*;
-
-#[cfg(feature = "botan3")]
-pub use pk_ops_kem::*;
-
-#[cfg(feature = "botan3")]
 pub use zfec::*;
+
+#[cfg(botan_ffi_20230403)]
+mod pk_ops_kem;
+
+#[cfg(botan_ffi_20230403)]
+pub use pk_ops_kem::*;
