@@ -34,6 +34,16 @@ impl MPI {
         self.obj
     }
 
+    pub(crate) fn handle_mut(&mut self) -> botan_mp_t {
+        self.obj
+    }
+
+    /// Ownership transfers to this MPI struct
+    #[allow(dead_code)]
+    pub(crate) fn from_handle(obj: botan_mp_t) -> Result<Self> {
+        Ok(MPI { obj })
+    }
+
     /// Crate a new (zero-valued) MPI
     pub fn new() -> Result<MPI> {
         let obj = botan_init!(botan_mp_init)?;
@@ -145,7 +155,7 @@ impl MPI {
                 botan_mp_to_str(self.obj, 16, out_buf as *mut c_char, out_len)
             })?;
 
-        if cfg!(feature = "botan3") {
+        if crate::Version::major_version() >= 3 {
             Ok(r.split_off(2)) // remove leading 0x
         } else {
             Ok(r)
@@ -329,7 +339,7 @@ impl MPI {
 
     /// Swap two MPI values
     pub fn swap(&mut self, other: &mut MPI) -> Result<()> {
-        botan_call!(botan_mp_swap, self.obj, other.obj)
+        botan_call!(botan_mp_swap, self.obj, other.handle_mut())
     }
 
     /// Perform a primality test on self
@@ -410,7 +420,7 @@ impl fmt::Debug for MPI {
     fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
         let s = self.to_string().map_err(|_| fmt::Error)?;
 
-        if cfg!(feature = "botan3") {
+        if crate::Version::major_version() >= 3 {
             write!(formatter, "{s}")
         } else {
             let is_positive = self.is_positive().map_err(|_| fmt::Error)?;
@@ -442,7 +452,7 @@ impl fmt::LowerHex for MPI {
     }
 }
 
-impl<'a> Add<&'a MPI> for MPI {
+impl Add<&MPI> for MPI {
     type Output = MPI;
 
     fn add(mut self, other: &MPI) -> MPI {
@@ -452,7 +462,7 @@ impl<'a> Add<&'a MPI> for MPI {
     }
 }
 
-impl<'a, 'b> Add<&'a MPI> for &'b MPI {
+impl Add<&MPI> for &MPI {
     type Output = MPI;
 
     fn add(self, other: &MPI) -> MPI {
@@ -470,7 +480,7 @@ impl Add<u32> for MPI {
     }
 }
 
-impl<'a> Add<u32> for &'a MPI {
+impl Add<u32> for &MPI {
     type Output = MPI;
 
     fn add(self, other: u32) -> MPI {
@@ -478,7 +488,7 @@ impl<'a> Add<u32> for &'a MPI {
     }
 }
 
-impl<'a> AddAssign<&'a MPI> for MPI {
+impl AddAssign<&MPI> for MPI {
     fn add_assign(&mut self, other: &MPI) {
         self.mp_add_assign(other)
             .expect("MPI::mp_add_assign succeeded");
@@ -492,7 +502,7 @@ impl AddAssign<u32> for MPI {
     }
 }
 
-impl<'a> Sub<&'a MPI> for MPI {
+impl Sub<&MPI> for MPI {
     type Output = MPI;
 
     fn sub(mut self, other: &MPI) -> MPI {
@@ -502,7 +512,7 @@ impl<'a> Sub<&'a MPI> for MPI {
     }
 }
 
-impl<'a, 'b> Sub<&'a MPI> for &'b MPI {
+impl Sub<&MPI> for &MPI {
     type Output = MPI;
 
     fn sub(self, other: &MPI) -> MPI {
@@ -520,7 +530,7 @@ impl Sub<u32> for MPI {
     }
 }
 
-impl<'a> Sub<u32> for &'a MPI {
+impl Sub<u32> for &MPI {
     type Output = MPI;
 
     fn sub(self, other: u32) -> MPI {
@@ -528,7 +538,7 @@ impl<'a> Sub<u32> for &'a MPI {
     }
 }
 
-impl<'a> SubAssign<&'a MPI> for MPI {
+impl SubAssign<&MPI> for MPI {
     fn sub_assign(&mut self, other: &MPI) {
         self.mp_sub_assign(other)
             .expect("MPI::mp_sub_assign succeeded");
@@ -542,7 +552,7 @@ impl SubAssign<u32> for MPI {
     }
 }
 
-impl<'a> Mul<&'a MPI> for MPI {
+impl Mul<&MPI> for MPI {
     type Output = MPI;
 
     fn mul(mut self, other: &MPI) -> MPI {
@@ -552,7 +562,7 @@ impl<'a> Mul<&'a MPI> for MPI {
     }
 }
 
-impl<'a, 'b> Mul<&'a MPI> for &'b MPI {
+impl Mul<&MPI> for &MPI {
     type Output = MPI;
 
     fn mul(self, other: &MPI) -> MPI {
@@ -560,14 +570,14 @@ impl<'a, 'b> Mul<&'a MPI> for &'b MPI {
     }
 }
 
-impl<'a> MulAssign<&'a MPI> for MPI {
+impl MulAssign<&MPI> for MPI {
     fn mul_assign(&mut self, other: &MPI) {
         self.mp_mul_assign(other)
             .expect("MPI::mp_mul_assign succeeded");
     }
 }
 
-impl<'a, 'b> Div<&'b MPI> for &'a MPI {
+impl Div<&MPI> for &MPI {
     type Output = MPI;
 
     #[inline]
@@ -583,7 +593,7 @@ impl<'a> DivAssign<&'a MPI> for MPI {
     }
 }
 
-impl<'a, 'b> Rem<&'b MPI> for &'a MPI {
+impl Rem<&MPI> for &MPI {
     type Output = MPI;
 
     fn rem(self, other: &MPI) -> MPI {
@@ -592,13 +602,13 @@ impl<'a, 'b> Rem<&'b MPI> for &'a MPI {
     }
 }
 
-impl<'a> RemAssign<&'a MPI> for MPI {
+impl RemAssign<&MPI> for MPI {
     fn rem_assign(&mut self, other: &MPI) {
         *self = &*self % other;
     }
 }
 
-impl<'a> Shl<usize> for &'a MPI {
+impl Shl<usize> for &MPI {
     type Output = MPI;
 
     fn shl(self, shift: usize) -> MPI {
@@ -613,7 +623,7 @@ impl ShlAssign<usize> for MPI {
     }
 }
 
-impl<'a> Shr<usize> for &'a MPI {
+impl Shr<usize> for &MPI {
     type Output = MPI;
 
     fn shr(self, shift: usize) -> MPI {
