@@ -13,7 +13,14 @@ unsafe impl Send for RandomNumberGenerator {}
 botan_impl_drop!(RandomNumberGenerator, botan_rng_destroy);
 
 impl RandomNumberGenerator {
-    fn new_of_type(typ: &str) -> Result<RandomNumberGenerator> {
+    /// Create a new RNG object with a specific type, e.g. "esdm-full"
+    ///
+    /// # Examples
+    /// ```
+    /// // This is just for demonstration, use RandomNumberGenerator::new_userspace() instead.
+    /// let specific_rng = botan::RandomNumberGenerator::new_of_type("user").unwrap();
+    /// ```
+    pub fn new_of_type(typ: &str) -> Result<RandomNumberGenerator> {
         let typ = make_cstr(typ)?;
         let obj = botan_init!(botan_rng_init, typ.as_ptr())?;
         Ok(RandomNumberGenerator { obj })
@@ -43,6 +50,45 @@ impl RandomNumberGenerator {
         RandomNumberGenerator::new_of_type("system")
     }
 
+    /// Create a new reference to the ESDM PRNG (fully seeded)
+    ///
+    /// Availability of this RNG depends on botan being compiled
+    /// with ESDM support.
+    ///
+    /// # Examples
+    /// ```ignore
+    /// let esdm_rng = botan::RandomNumberGenerator::new_esdm().unwrap();
+    /// ```
+    pub fn new_esdm() -> Result<RandomNumberGenerator> {
+        RandomNumberGenerator::new_of_type("esdm-full")
+    }
+
+    /// Create a new reference to the ESDM PRNG (with prediction resistance)
+    ///
+    /// Availability of this RNG depends on botan being compiled
+    /// with ESDM support.
+    ///
+    /// # Examples
+    /// ```ignore
+    /// let esdm_rng = botan::RandomNumberGenerator::new_esdm_pr().unwrap();
+    /// ```
+    pub fn new_esdm_pr() -> Result<RandomNumberGenerator> {
+        RandomNumberGenerator::new_of_type("esdm-pr")
+    }
+
+    /// Create a new reference to the Jitter RNG
+    ///
+    /// Availability of this RNG depends on botan being compiled
+    /// with Jitter RNG support.
+    ///
+    /// # Examples
+    /// ```ignore
+    /// let jitter_rng = botan::RandomNumberGenerator::new_jitter().unwrap();
+    /// ```
+    pub fn new_jitter() -> Result<RandomNumberGenerator> {
+        RandomNumberGenerator::new_of_type("jitter")
+    }
+
     /// Create a new reference to an RNG of some arbitrary type
     ///
     /// # Examples
