@@ -1,16 +1,16 @@
 use botan_sys::*;
 use core::fmt;
 
-#[cfg(feature = "no-std")]
+#[cfg(not(feature = "std"))]
 pub(crate) use alloc::{borrow::ToOwned, string::String, string::ToString, vec::Vec};
 
-#[cfg(feature = "no-std")]
+#[cfg(not(feature = "std"))]
 pub(crate) use alloc::ffi::CString;
 
-#[cfg(feature = "no-std")]
+#[cfg(not(feature = "std"))]
 pub(crate) use core::ffi::CStr;
 
-#[cfg(not(feature = "no-std"))]
+#[cfg(feature = "std")]
 pub(crate) use std::ffi::{CStr, CString};
 
 pub(crate) use botan_sys::ffi_types::{c_char, c_int, c_void};
@@ -52,14 +52,14 @@ pub(crate) fn call_botan_ffi_returning_vec_u8(
     Ok(output)
 }
 
-#[cfg(feature = "botan3")]
+#[cfg(botan_ffi_20230403)]
 pub(crate) mod view {
     use super::*;
 
     type FfiViewBinaryFn = extern "C" fn(*mut c_void, *const u8, usize) -> c_int;
 
     extern "C" fn botan_ffi_view_u8_fn(ctx: *mut c_void, buf: *const u8, len: usize) -> c_int {
-        if ctx == core::ptr::null_mut() || buf == core::ptr::null_mut() {
+        if ctx.is_null() || buf.is_null() {
             return BOTAN_FFI_ERROR_NULL_POINTER;
         }
 
@@ -68,7 +68,7 @@ pub(crate) mod view {
         unsafe {
             let data = core::slice::from_raw_parts(buf, len);
             (*vec).clear();
-            (*vec).extend_from_slice(&data);
+            (*vec).extend_from_slice(data);
         }
 
         0
@@ -92,7 +92,7 @@ pub(crate) mod view {
     type FfiViewStrFn = extern "C" fn(*mut c_void, *const c_char, usize) -> c_int;
 
     extern "C" fn botan_ffi_view_str_fn(ctx: *mut c_void, buf: *const c_char, len: usize) -> c_int {
-        if ctx == core::ptr::null_mut() || buf == core::ptr::null_mut() {
+        if ctx.is_null() || buf.is_null() {
             return BOTAN_FFI_ERROR_NULL_POINTER;
         }
 
@@ -105,7 +105,7 @@ pub(crate) mod view {
         let data = unsafe { core::slice::from_raw_parts(buf as *const u8, len - 1) };
 
         let mut vec = Vec::new();
-        vec.extend_from_slice(&data);
+        vec.extend_from_slice(data);
         match String::from_utf8(vec) {
             Ok(decoded) => {
                 unsafe {
@@ -133,7 +133,7 @@ pub(crate) mod view {
     }
 }
 
-#[cfg(feature = "botan3")]
+#[cfg(botan_ffi_20230403)]
 pub(crate) use crate::view::*;
 
 fn cstr_slice_to_str(raw_cstr: &[u8]) -> Result<String> {
@@ -141,12 +141,26 @@ fn cstr_slice_to_str(raw_cstr: &[u8]) -> Result<String> {
     Ok(cstr.to_str().map_err(Error::conversion_error)?.to_owned())
 }
 
-#[cfg(feature = "botan3")]
+#[cfg(botan_ffi_20230403)]
 unsafe fn cstr_to_str(raw_cstr: *const c_char) -> Result<String> {
     let cstr = CStr::from_ptr(raw_cstr);
     Ok(cstr.to_str().map_err(Error::conversion_error)?.to_owned())
 }
 
+#[cfg(botan_ffi_20250506)]
+pub(crate) fn interp_as_bool(result: c_int, fn_name: &'static str) -> Result<bool> {
+    if result == 0 {
+        Ok(false)
+    } else if result == 1 {
+        Ok(true)
+    } else {
+        Err(Error::with_message(
+            ErrorType::InternalError,
+            format!("Unexpected result from {}", fn_name),
+        ))
+    }
+}
+
 pub(crate) fn call_botan_ffi_returning_string(
     initial_size: usize,
     cb: &dyn Fn(*mut u8, *mut usize) -> c_int,
@@ -155,6 +169,43 @@ pub(crate) fn call_botan_ffi_returning_string(
     cstr_slice_to_str(&v)
 }
 
+#[allow(unused_macros)]
+macro_rules! ffi_version_from_cfg {
+    (botan_ffi_20230403) => {
+        20230403
+    };
+    (botan_ffi_20240408) => {
+        20240408
+    };
+    (botan_ffi_20250506) => {
+        20250506
+    };
+}
+
+pub(crate) use ffi_version_from_cfg;
+
+macro_rules! ffi_version_guard {
+    ($fn_name:expr, $cfg_val:ident, [ $($arg:ident),* ], $if_impl:block) => {{
+        #[cfg($cfg_val)]
+        {
+            $if_impl
+        }
+
+        #[cfg(not($cfg_val))]
+        {
+            $(
+                let _ = $arg;
+            )*
+            Err(Error::not_implemented(
+                $fn_name,
+                ffi_version_from_cfg!($cfg_val),
+            ))
+        }
+    }};
+}
+
+pub(crate) use ffi_version_guard;
+
 /// The library error type
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub struct Error {
@@ -179,17 +230,17 @@ impl Error {
     pub(crate) fn from_rc(rc: c_int) -> Self {
         let err_type = ErrorType::from(rc);
 
-        #[cfg(feature = "botan3")]
+        #[cfg(botan_ffi_20230403)]
         let message = {
             let cptr = unsafe { botan_sys::botan_error_last_exception_message() };
             match unsafe { cstr_to_str(cptr) } {
                 Err(_) => None,
-                Ok(s) if s.len() > 0 => Some(s),
+                Ok(s) if !s.is_empty() => Some(s),
                 Ok(_) => None,
             }
         };
 
-        #[cfg(not(feature = "botan3"))]
+        #[cfg(not(botan_ffi_20230403))]
         let message = None;
 
         Self { err_type, message }
@@ -202,7 +253,19 @@ impl Error {
         }
     }
 
-    #[cfg(not(feature = "no-std"))]
+    pub(crate) fn bad_parameter(message: &'static str) -> Self {
+        Self::with_message(ErrorType::BadParameter, message.to_owned())
+    }
+
+    #[allow(dead_code)]
+    pub(crate) fn not_implemented(fn_name: &'static str, ffi_version: u32) -> Self {
+        Self::with_message(
+            ErrorType::NotImplemented,
+            format!("Function {fn_name} not available - requires Botan FFI {ffi_version}"),
+        )
+    }
+
+    #[cfg(feature = "std")]
     pub(crate) fn conversion_error<T: std::error::Error>(e: T) -> Self {
         Self {
             err_type: ErrorType::ConversionError,
@@ -211,7 +274,7 @@ impl Error {
     }
 
     // Hack to deal with missing std::error::Error in no-std
-    #[cfg(feature = "no-std")]
+    #[cfg(not(feature = "std"))]
     pub(crate) fn conversion_error<T: core::fmt::Display>(e: T) -> Self {
         Self {
             err_type: ErrorType::ConversionError,
@@ -238,6 +301,8 @@ pub enum ErrorType {
     BadFlag,
     /// An invalid parameter was provided to the library
     BadParameter,
+    /// No value available
+    NoValueAvailable,
     /// An exception was thrown while processing this request
     ExceptionThrown,
     /// There was insufficient buffer space to write the output
@@ -282,6 +347,7 @@ impl fmt::Display for ErrorType {
             Self::BadAuthCode => "A provided authentication code was incorrect",
             Self::BadFlag => "A bad flag was passed to the library",
             Self::BadParameter => "An invalid parameter was provided to the library",
+            Self::NoValueAvailable => "No value was available",
             Self::ExceptionThrown => "An exception was thrown while processing this request",
             Self::StringConversionError => "Error converting a string into UTF-8",
             Self::InsufficientBufferSpace => {
@@ -312,7 +378,7 @@ impl fmt::Display for ErrorType {
     }
 }
 
-#[cfg(not(feature = "no-std"))]
+#[cfg(feature = "std")]
 impl std::error::Error for Error {}
 
 impl From<i32> for ErrorType {
@@ -321,6 +387,7 @@ impl From<i32> for ErrorType {
             BOTAN_FFI_ERROR_BAD_FLAG => Self::BadFlag,
             BOTAN_FFI_ERROR_BAD_MAC => Self::BadAuthCode,
             BOTAN_FFI_ERROR_BAD_PARAMETER => Self::BadParameter,
+            BOTAN_FFI_ERROR_NO_VALUE => Self::NoValueAvailable,
             BOTAN_FFI_ERROR_EXCEPTION_THROWN => Self::ExceptionThrown,
             BOTAN_FFI_ERROR_HTTP_ERROR => Self::HttpError,
             BOTAN_FFI_ERROR_INSUFFICIENT_BUFFER_SPACE => Self::InsufficientBufferSpace,
