@@ -82,7 +82,7 @@ fn wycheproof_keywrap_tests() -> Result<(), botan::Error> {
     Ok(())
 }
 
-#[cfg(feature = "botan3")]
+#[cfg(botan_ffi_20230403)]
 #[test]
 fn wycheproof_nist_kw_tests() -> Result<(), botan::Error> {
     use wycheproof::keywrap::*;
@@ -112,7 +112,7 @@ fn wycheproof_nist_kw_tests() -> Result<(), botan::Error> {
 
                 (Algorithm::SeedKeyWrap, 128) => ("SEED", false),
 
-                (bc, kl) => panic!("Unhandled block cipher {:?}/{}", bc, kl),
+                (bc, kl) => panic!("Unhandled block cipher {bc:?}/{kl}"),
             };
 
             for test in &test_group.tests {
@@ -206,12 +206,12 @@ fn wycheproof_aead_gcm_tests() -> Result<(), botan::Error> {
 
     fn seed_gcm_name(ks: usize, ts: usize, _ns: usize) -> Option<String> {
         assert!(ks == 128 && ts == 128);
-        Some(format!("SEED/GCM"))
+        Some("SEED/GCM".to_string())
     }
 
     fn sm4_gcm_name(ks: usize, ts: usize, _ns: usize) -> Option<String> {
         assert!(ks == 128 && ts == 128);
-        Some(format!("SM4/GCM"))
+        Some("SM4/GCM".to_string())
     }
 
     wycheproof_aead_test(wycheproof::aead::TestName::AesGcm, aes_gcm_name)?;
@@ -235,7 +235,7 @@ fn wycheproof_aead_ccm_tests() -> Result<(), botan::Error> {
             return None;
         }
         let ccm_l = 15 - (ns / 8);
-        if ccm_l < 2 || ccm_l > 8 {
+        if !(2..=8).contains(&ccm_l) {
             return None;
         }
         Some(format!("AES-{ks}/CCM({tag_bytes},{ccm_l})"))
@@ -423,7 +423,7 @@ fn wycheproof_mac_tests() -> Result<(), botan::Error> {
 
     wycheproof_mac_test(TestName::AesCmac, |ks: usize| {
         if ks == 128 || ks == 192 || ks == 256 {
-            Some(format!("CMAC(AES-{})", ks))
+            Some(format!("CMAC(AES-{ks})"))
         } else {
             None
         }
@@ -431,7 +431,7 @@ fn wycheproof_mac_tests() -> Result<(), botan::Error> {
 
     wycheproof_mac_test(TestName::AriaCmac, |ks: usize| {
         if ks == 128 || ks == 192 || ks == 256 {
-            Some(format!("CMAC(ARIA-{})", ks))
+            Some(format!("CMAC(ARIA-{ks})"))
         } else {
             None
         }
@@ -439,7 +439,7 @@ fn wycheproof_mac_tests() -> Result<(), botan::Error> {
 
     wycheproof_mac_test(TestName::CamelliaCmac, |ks: usize| {
         if ks == 128 || ks == 192 || ks == 256 {
-            Some(format!("CMAC(Camellia-{})", ks))
+            Some(format!("CMAC(Camellia-{ks})"))
         } else {
             None
         }
@@ -482,7 +482,7 @@ fn wycheproof_mac_test(
     Ok(())
 }
 
-#[cfg(feature = "botan3")]
+#[cfg(botan_ffi_20230403)]
 #[test]
 fn wycheproof_mac_with_nonce_tests() -> Result<(), botan::Error> {
     use wycheproof::mac_with_nonce::*;
@@ -512,6 +512,7 @@ fn wycheproof_mac_with_nonce_tests() -> Result<(), botan::Error> {
 
     Ok(())
 }
+
 #[test]
 fn wycheproof_primality_tests() -> Result<(), botan::Error> {
     use wycheproof::{primality::*, TestResult};
@@ -529,7 +530,7 @@ fn wycheproof_primality_tests() -> Result<(), botan::Error> {
 
                 // The primality test data encodes negative numbers using
                 // twos complement encoding
-                let mpi = if test.value.len() > 0 && (test.value[0] & 0x80 == 0x80) {
+                let mpi = if !test.value.is_empty() && (test.value[0] & 0x80 == 0x80) {
                     let mut flipped: Vec<u8> = test.value.to_vec();
                     for i in 0..flipped.len() {
                         flipped[i] = !flipped[i];
@@ -593,17 +594,11 @@ fn wycheproof_rsa_oaep_decrypt_tests() -> Result<(), botan::Error> {
             return None;
         }
 
-        let label_hash = match hash_id_to_str(group.hash) {
-            Some(h) => h,
-            None => return None,
-        };
+        let label_hash = hash_id_to_str(group.hash)?;
 
-        let mgf_hash = match hash_id_to_str(group.mgf_hash) {
-            Some(h) => h,
-            None => return None,
-        };
+        let mgf_hash = hash_id_to_str(group.mgf_hash)?;
 
-        Some(format!("OAEP({},MGF1({}))", label_hash, mgf_hash))
+        Some(format!("OAEP({label_hash},MGF1({mgf_hash}))"))
     }
 
     let is_botan2 = botan::Version::current()?.major == 2;
@@ -612,7 +607,7 @@ fn wycheproof_rsa_oaep_decrypt_tests() -> Result<(), botan::Error> {
         let test_set = TestSet::load(test_name).expect("Loading tests failed");
 
         for test_group in &test_set.test_groups {
-            let oaep_string = match gen_oaep_string(&test_group) {
+            let oaep_string = match gen_oaep_string(test_group) {
                 Some(s) => s,
                 None => continue,
             };
@@ -661,7 +656,7 @@ fn wycheproof_rsa_pkcs1_verify_tests() -> Result<(), botan::Error> {
 
             let key = botan::Pubkey::load_der(&test_group.der)?;
 
-            let mut verifier = botan::Verifier::new(&key, &format!("EMSA_PKCS1({})", hash))?;
+            let mut verifier = botan::Verifier::new(&key, &format!("EMSA_PKCS1({hash})"))?;
 
             for test in &test_group.tests {
                 verifier.update(&test.msg)?;
@@ -700,10 +695,7 @@ fn wycheproof_rsa_pss_verify_tests() -> Result<(), botan::Error> {
             (_, _, _) => return None,
         }
 
-        let hash = match hash_id_to_str(group.hash) {
-            Some(hash) => hash,
-            None => return None,
-        };
+        let hash = hash_id_to_str(group.hash)?;
 
         Some(format!("EMSA4({},MGF1,{})", hash, group.salt_size))
     }
@@ -753,7 +745,7 @@ fn wycheproof_dsa_verify_tests() -> Result<(), botan::Error> {
     use wycheproof::dsa::*;
 
     for test_name in TestName::all() {
-        let is_ieee = format!("{:?}", test_name).contains("P1363");
+        let is_ieee = format!("{test_name:?}").contains("P1363");
         let test_set = TestSet::load(test_name).expect("Loading tests failed");
 
         for test_group in &test_set.test_groups {
@@ -768,11 +760,11 @@ fn wycheproof_dsa_verify_tests() -> Result<(), botan::Error> {
                 // Has to be inside the loop to work around the bug addressed in
                 // https://github.com/randombit/botan/pull/3333
                 let mut verifier = if is_ieee {
-                    botan::Verifier::new(&key, &format!("EMSA1({})", hash))?
+                    botan::Verifier::new(&key, &format!("EMSA1({hash})"))?
                 } else {
                     botan::Verifier::new_with_der_formatted_signatures(
                         &key,
-                        &format!("EMSA1({})", hash),
+                        &format!("EMSA1({hash})"),
                     )?
                 };
 
@@ -837,13 +829,13 @@ fn wycheproof_ecdsa_verify_tests() -> Result<(), botan::Error> {
             continue;
         }
 
-        let is_ieee = format!("{:?}", test_name).contains("P1363")
-            || format!("{:?}", test_name).contains("Webcrypto");
+        let is_ieee = format!("{test_name:?}").contains("P1363")
+            || format!("{test_name:?}").contains("Webcrypto");
 
         let test_set = TestSet::load(test_name).expect("Loading tests failed");
 
         for test_group in &test_set.test_groups {
-            if curve_id_to_str(test_group.key.curve) == None {
+            if curve_id_to_str(test_group.key.curve).is_none() {
                 continue;
             }
 
@@ -865,7 +857,7 @@ fn wycheproof_ecdsa_verify_tests() -> Result<(), botan::Error> {
 
             let key = botan::Pubkey::load_der(&test_group.der)?;
 
-            let mut verifier_ieee = botan::Verifier::new(&key, &format!("EMSA1({})", hash))?;
+            let mut verifier_ieee = botan::Verifier::new(&key, &format!("EMSA1({hash})"))?;
 
             for test in &test_group.tests {
                 let accept = if is_ieee {
@@ -876,7 +868,7 @@ fn wycheproof_ecdsa_verify_tests() -> Result<(), botan::Error> {
                     // https://github.com/randombit/botan/pull/3333
                     let mut verifier_der = botan::Verifier::new_with_der_formatted_signatures(
                         &key,
-                        &format!("EMSA1({})", hash),
+                        &format!("EMSA1({hash})"),
                     )?;
 
                     verifier_der.update(&test.msg)?;
@@ -909,9 +901,10 @@ fn wycheproof_eddsa_verify_tests() -> Result<(), botan::Error> {
     use wycheproof::eddsa::*;
 
     let is_botan2 = botan::Version::current()?.major == 2;
+    let supports_ed448 = cfg!(botan_ffi_20240408);
 
     for test_name in TestName::all() {
-        if test_name == TestName::Ed448 {
+        if test_name == TestName::Ed448 && !supports_ed448 {
             continue;
         }
 
@@ -975,7 +968,7 @@ fn wycheproof_ecdh_tests() -> Result<(), botan::Error> {
 
             for test in &test_group.tests {
                 let s = botan::MPI::new_from_bytes(&test.private_key)?;
-                let priv_key = botan::Privkey::load_ecdh(&s, &curve_id)?;
+                let priv_key = botan::Privkey::load_ecdh(&s, curve_id)?;
 
                 let mut ka = botan::KeyAgreement::new(&priv_key, "Raw")?;
 
@@ -984,7 +977,7 @@ fn wycheproof_ecdh_tests() -> Result<(), botan::Error> {
                 if test.result == TestResult::Valid || test.result == TestResult::Acceptable {
                     match shared_secret {
                         Ok(shared_secret) => assert_eq!(shared_secret, test.shared_secret.as_ref()),
-                        Err(e) => panic!("Unable to compute shared secret ({:?})", e),
+                        Err(e) => panic!("Unable to compute shared secret ({e:?})"),
                     }
                 } else {
                     assert!(shared_secret.is_err());
@@ -1000,8 +993,13 @@ fn wycheproof_ecdh_tests() -> Result<(), botan::Error> {
 fn wycheproof_xdh_tests() -> Result<(), botan::Error> {
     use wycheproof::xdh::*;
 
+    let version = botan::Version::current()?;
+    let will_accept_zeros = !version.at_least(3, 6);
+
+    let supports_x448 = botan::Version::supports_version(20240408);
+
     for test_name in TestName::all() {
-        if test_name == TestName::X448 {
+        if test_name == TestName::X448 && !supports_x448 {
             continue;
         }
 
@@ -1009,16 +1007,32 @@ fn wycheproof_xdh_tests() -> Result<(), botan::Error> {
 
         for test_group in &test_set.test_groups {
             for test in &test_group.tests {
-                let priv_key = botan::Privkey::load_x25519(&test.private_key)?;
+                let priv_key = match test_name {
+                    #[cfg(botan_ffi_20240408)]
+                    TestName::X448 => botan::Privkey::load_x448(&test.private_key),
+                    _ => botan::Privkey::load_x25519(&test.private_key),
+                }?;
 
                 let mut ka = botan::KeyAgreement::new(&priv_key, "Raw")?;
 
                 let shared_secret = ka.agree(0, &test.public_key, &[]);
 
-                if test.result == TestResult::Valid || test.result == TestResult::Acceptable {
+                let should_accept = match test.result {
+                    TestResult::Valid => true,
+                    TestResult::Acceptable => {
+                        if test.flags.contains(&TestFlag::ZeroSharedSecret) {
+                            will_accept_zeros
+                        } else {
+                            true
+                        }
+                    }
+                    TestResult::Invalid => false,
+                };
+
+                if should_accept {
                     match shared_secret {
                         Ok(shared_secret) => assert_eq!(shared_secret, test.shared_secret.as_ref()),
-                        Err(e) => panic!("Unable to compute shared secret ({:?})", e),
+                        Err(e) => panic!("Unable to compute shared secret ({e:?})"),
                     }
                 } else {
                     assert!(shared_secret.is_err());
